@@ -1,6 +1,4 @@
 # encoding: utf-8
-from bzrlib import remote
-from errno import EPIPE
 import os
 import sys
 import platform
