@@ -9,12 +9,19 @@ from emq.range.constants import GALAXY_EMQ_QUEUE_DELAY_SECONDS_MINIMAL, GALAXY_E
   GALAXY_EMQ_QUEUE_MAX_MESSAGE_BYTES_MINIMAL, GALAXY_EMQ_QUEUE_MAX_MESSAGE_BYTES_MAXIMAL, \
   GALAXY_EMQ_QUEUE_PARTITION_NUMBER_MINIMAL, GALAXY_EMQ_QUEUE_PARTITION_NUMBER_MAXIMAL, \
   GALAXY_EMQ_MESSAGE_DELAY_SECONDS_MAXIMAL, GALAXY_EMQ_MESSAGE_DELAY_SECONDS_MINIMAL, \
-  GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MAXIMAL, GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MINIMAL
+  GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MAXIMAL, GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MINIMAL, \
+  GALAXY_EMQ_QUEUE_WRITE_QPS_MINIMAL, GALAXY_EMQ_QUEUE_WRITE_QPS_MAXIMAL, GALAXY_EMQ_QUEUE_READ_QPS_MINIMAL, \
+  GALAXY_EMQ_QUEUE_READ_QPS_MAXIMAL, GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MINIMAL, GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MAXIMAL, \
+  GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MINIMAL, GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MAXIMAL
 from emq.message.ttypes import SendMessageRequest, ReceiveMessageRequest, ChangeMessageVisibilityRequest, \
   DeleteMessageRequest, SendMessageBatchRequest, SendMessageBatchRequestEntry, ChangeMessageVisibilityBatchRequestEntry, \
   ChangeMessageVisibilityBatchRequest, DeleteMessageBatchRequest
 from emq.queue.ttypes import CreateQueueRequest, ListQueueRequest, SetQueueAttributesRequest, SetPermissionRequest, \
-  RevokePermissionRequest, QueryPermissionForIdRequest
+  RevokePermissionRequest, QueryPermissionForIdRequest, SetQueueQuotaRequest, QueueQuota, CreateTagRequest, \
+  DeleteTagRequest, GetTagInfoRequest, ListTagRequest, SetQueueRedrivePolicyRequest, RemoveQueueRedrivePolicyRequest, \
+  ListDeadLetterSourceQueuesRequest
+from emq.statistics.ttypes import SetUserQuotaRequest, GetUserQuotaRequest, GetUserUsedQuotaRequest, SetUserInfoRequest, \
+  GetUserInfoRequest
 
 
 class RequestChecker(object):
@@ -34,9 +41,20 @@ class RequestChecker(object):
     elif isinstance(request, CreateQueueRequest):
       self.validate_queue_name(request.queueName, False)
       self.validate_queue_attribute(request.queueAttribute)
+      self.validate_queue_quota(request.queueQuota)
     elif isinstance(request, SetQueueAttributesRequest):
       self.validate_queue_name(request.queueName)
       self.validate_queue_attribute(request.queueAttribute)
+    elif isinstance(request, SetQueueQuotaRequest):
+      self.validate_queue_name(request.queueName)
+      self.validate_queue_quota(request.queueQuota)
+    elif isinstance(request, SetQueueRedrivePolicyRequest):
+      self.validate_queue_name(request.queueName)
+      self.validate_redrivePolcy(request.redrivePolicy)
+    elif isinstance(request, RemoveQueueRedrivePolicyRequest):
+      self.validate_queue_name(request.queueName)
+    elif isinstance(request, ListDeadLetterSourceQueuesRequest):
+      self.validate_queue_name(request.dlqName)
     elif isinstance(request, SetPermissionRequest):
       self.validate_queue_name(request.queueName)
       self.validate_not_none(request.developerId, "developerId")
@@ -47,12 +65,31 @@ class RequestChecker(object):
     elif isinstance(request, QueryPermissionForIdRequest):
       self.validate_queue_name(request.queueName)
       self.validate_not_none(request.developerId, "developerId")
+    elif isinstance(request, CreateTagRequest):
+      self.validate_queue_name(request.queueName)
+      self.validate_queue_name(request.tagName, False, False, "tag name")
+      if request.attributeName:
+        self.validate_not_empty(request.attributeName, "attributeName")
+        self.check_message_attribute(request.attributeValue, True)
+      if request.userAttributes:
+        self.validate_user_attribute(request.userAttributes)
+      if request.readQPSQuota:
+        self.validate_readQps(request.readQPSQuota)
+    elif isinstance(request, DeleteTagRequest):
+      self.validate_queue_name(request.queueName)
+      self.validate_queue_name(request.tagName, False, False, "tag name")
+    elif isinstance(request, GetTagInfoRequest):
+      self.validate_queue_name(request.queueName)
+      if request.tagName is not None:
+        self.validate_queue_name(request.tagName, False, False, "tag name")
+    elif isinstance(request, ListTagRequest):
+      self.validate_queue_name(request.queueName)
     elif isinstance(request, SendMessageRequest):
       self.validate_queue_name(request.queueName)
       self.validate_not_none(request.messageBody, "messageBody")
       if request.messageAttributes is not None:
         for attribute in request.messageAttributes.values():
-          self.check_message_attribute(attribute)
+          self.check_message_attribute(attribute, False)
       if request.delaySeconds is not None:
         self.validate_delaySeconds(request.delaySeconds)
       if request.invisibilitySeconds is not None:
@@ -63,6 +100,11 @@ class RequestChecker(object):
         self.validate_receiveMessageMaximumNumber(request.maxReceiveMessageNumber)
       if request.maxReceiveMessageWaitSeconds is not None:
         self.validate_receiveMessageWaitSeconds(request.maxReceiveMessageWaitSeconds)
+      if request.attributeName is not None:
+        self.validate_not_empty(request.attributeName, "attributeName")
+        self.check_message_attribute(request.attributeValue, True)
+      if request.tagName is not None:
+        self.validate_queue_name(request.tagName, False, False, "tag name")
     elif isinstance(request, ChangeMessageVisibilityRequest):
       self.validate_queue_name(request.queueName)
       if request.invisibilitySeconds is not None:
@@ -87,7 +129,8 @@ class RequestChecker(object):
     elif isinstance(request, ChangeMessageVisibilityBatchRequest):
       self.validate_queue_name(request.queueName)
       changeMessageVisibilityBatchRequestEntryList = request.changeMessageVisibilityRequestEntryList
-      self.validate_not_empty(changeMessageVisibilityBatchRequestEntryList, "changeMessageVisibilityBatchRequestEntryList")
+      self.validate_not_empty(changeMessageVisibilityBatchRequestEntryList,
+                              "changeMessageVisibilityBatchRequestEntryList")
       receipt_handle_list = []
       for k in changeMessageVisibilityBatchRequestEntryList:
         self.validate_not_none(k.receiptHandle, "receiptHandle")
@@ -105,6 +148,10 @@ class RequestChecker(object):
         self.validate_not_empty(k.receiptHandle, "receiptHandle")
         receipt_handle_list.append(k.receiptHandle)
       self.check_list_duplicate(receipt_handle_list, "receiptHandle")
+    elif (isinstance(request, SetUserQuotaRequest) or isinstance(request, GetUserQuotaRequest)
+      or isinstance(request, GetUserUsedQuotaRequest) or isinstance(request, SetUserInfoRequest)
+      or isinstance(request, GetUserInfoRequest)):
+      pass
     else:
       self.validate_queue_name(request.queueName)
 
@@ -116,15 +163,18 @@ class RequestChecker(object):
       self.validate_invisibilitySeconds(send_entry.invisibilitySeconds)
     if send_entry.messageAttributes is not None:
       for attribute in send_entry.messageAttributes.values():
-        self.check_message_attribute(attribute)
+        self.check_message_attribute(attribute, False)
 
-  def check_message_attribute(self, attribute):
+  def check_message_attribute(self, attribute, allow_empty):
+    self.validate_not_none(attribute, "messageAttribute")
     if attribute.type.lower().startswith("string"):
       if attribute.stringValue is None:
         raise GalaxyEmqServiceException(errMsg="stringValue cannot be None when type is STRING")
     elif attribute.type.lower().startswith("binary"):
       if attribute.binaryValue is None:
         raise GalaxyEmqServiceException(errMsg="binaryValue cannot be None when type is BINARY")
+    elif allow_empty and attribute.type.lower() == "empty":
+      return
     else:
       raise GalaxyEmqServiceException(errMsg="Attribute type must start with \"STRING\" or \"BINARY\"")
     for c in attribute.type:
@@ -189,6 +239,23 @@ class RequestChecker(object):
       self.validate_messageMaximumBytes(queue_attribute.messageMaximumBytes)
     if queue_attribute.partitionNumber:
       self.validate_partitionNumber(queue_attribute.partitionNumber)
+    if queue_attribute.userAttributes:
+      self.validate_user_attribute(queue_attribute.userAttributes)
+
+  def validate_user_attribute(self, attribute):
+    for key, value in attribute.items():
+      self.validate_not_empty(key, "user attribute name")
+      self.validate_not_empty(value, "user attribute value for \"" + key + "\"")
+
+  def validate_queue_quota(self, queue_quota):
+    if not queue_quota:
+      return
+    if not queue_quota.throughput:
+      return
+    if queue_quota.throughput.readQps:
+      self.validate_readQps(queue_quota.throughput.readQps)
+    if queue_quota.throughput.writeQps:
+      self.validate_writeQps(queue_quota.throughput.writeQps)
 
   def check_filed_range(self, value, low, high, name):
     if not isinstance(value, int):
@@ -255,11 +322,34 @@ class RequestChecker(object):
                            GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MAXIMAL,
                            "messageInvisibilitySeconds")
 
+  def validate_spaceQuota(self, spaceQuota):
+    self.check_filed_range(spaceQuota,
+                           GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MINIMAL,
+                           GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MAXIMAL,
+                           "spaceQuota")
+
+  def validate_readQps(self, readQps):
+    self.check_filed_range(readQps,
+                           GALAXY_EMQ_QUEUE_READ_QPS_MINIMAL,
+                           GALAXY_EMQ_QUEUE_READ_QPS_MAXIMAL,
+                           "readQps")
+
+  def validate_writeQps(self, writeQps):
+    self.check_filed_range(writeQps,
+                           GALAXY_EMQ_QUEUE_WRITE_QPS_MINIMAL,
+                           GALAXY_EMQ_QUEUE_WRITE_QPS_MAXIMAL,
+                           "writeQps")
+  def validate_redrivePolcy(self, redrivePolicy):
+    self.validate_queue_name(redrivePolicy.dlqName)
+    self.check_filed_range(redrivePolicy.maxReceiveTime,
+                           GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MINIMAL,
+                           GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MAXIMAL)
+
   def validate_not_none(self, param, name):
     if param is None:
       raise GalaxyEmqServiceException(errMsg="Bad request, the %s is required!" % name)
 
   def validate_not_empty(self, param, name):
     if not param:
-        raise GalaxyEmqServiceException(errMsg="Bad request, the %s shouldn't be empty!" % name)
+      raise GalaxyEmqServiceException(errMsg="Bad request, the %s shouldn't be empty!" % name)
 
